local addon, data = ...

local math_sqrt = math.sqrt
local math_atan = math.atan2
local math_pi = math.pi
local string_format = string.format
local math_max = math.max

local INSPECT_QUEST_DETAIL = {}
local INSPECT_QUEST_LIST = {}
local INSPECT_UNIT_DETAIL = {}

_ONSLAUGHT = { UI={} }

local OS = _ONSLAUGHT

local _QID_ONS = { cnt=0, list = {} }
local _QID_ALL = {}
local _QID_DTL = {}

OS.ALL = _QID_DTL

local _currentZone = nil
local _cacheZone = {}

local ONS_QSTDTL = false
local ONS_DTLCOMP = false

local ONS_GPSUPDATE = false

local GPSDATA = {}
local GPSDXCNT = 0

local GPS_MODE_ALL = 1
local GPS_MODE_AVAIL=2

local GPS_TRACK_QST = 1
local GPS_TRACK_OBJ = 2

local GPS_VISIBLE = false

local default_settings = {
	mmx = 200, mmy = 200, locked = false, autoswap = true,
	details = {}, gpsscale = 1, gpsmode = GPS_MODE_AVAIL, usegps = true,
	gpsx = 600, gpsy = 200, gpstrack = GPS_TRACK_QST, fonts = { label = 12, dist = 8}
	}
	
local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}			
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end
	
function OS.GPS_Scale()
	local h = 0
	
	uh_fw = OS.UI.gps_dims.h_wd
	
	local lw = 0
	local dw = 0
	
	for x=1, 25 do
		OS.UI.gps_tracker[x].n:SetFontSize(Onslaught_Settings.fonts.label)
		OS.UI.gps_tracker[x].d:SetFontSize(Onslaught_Settings.fonts.dist)
		h = OS.UI.gps_tracker[x].n:GetHeight() + OS.UI.gps_tracker[x].d:GetHeight()
		OS.UI.gps_tracker[x].i:SetHeight(h)
		OS.UI.gps_tracker[x].i:SetWidth(h)
		OS.UI.gps_tracker[x].f:SetHeight(h)
		if x <= GPSDXCNT then
			uh_fw = math.max(uh_fw, math.max(OS.UI.gps_tracker[x].n:GetWidth(), OS.UI.gps_tracker[x].d:GetWidth())+h)
		end
	end
	
	OS.UI.gps_dims.i_ht = h
	OS.UI.gps_dims.n_ht = OS.UI.gps_tracker[1].n:GetHeight()
	OS.UI.gps_dims.d_ht = OS.UI.gps_tracker[1].d:GetHeight()
	
	OS.UI.gps:SetWidth(4+uh_fw)
	OS.UI.gps_header:SetWidth(uh_fw)
	OS.UI.gps_config:SetWidth(uh_fw)
	OS.UI.gps:SetHeight(4+(GPSDXCNT*(OS.UI.gps_dims.n_ht+OS.UI.gps_dims.d_ht))+OS.UI.gps_dims.h_ht+OS.UI.gps_dims.m_ht)
end	
	
function OS.CheckDay()
	local tdy = os.date("%Y-%m-%d", Inspect.Time.Server()-(60*60*4))
	if Onslaught_Settings.day ~= tdy then
		Onslaught_Settings.day = tdy
		Onslaught_Settings.complete = {}
	end
end

function OS.HeaderClick()
	Onslaught_Settings.usegps = not Onslaught_Settings.usegps
	print(string_format("Using GPS: %s", string.upper(tostring(Onslaught_Settings.usegps))))
	if data.quests[_currentZone] and Onslaught_Settings.usegps then
		OS.UI.gps:SetVisible(true)
		GPS_VISIBLE = true
	else
		OS.UI.gps:SetVisible(false)
		GPS_VISIBLE = false
	end
	--OS.GPS()
end

function OS.BuildUI()
	OS.context = UI.CreateContext(addon.identifier)
	OS.UI.mm = UI.CreateFrame("Texture", "OS.UI.mm", OS.context)
	OS.UI.mm:SetHeight(36)
	OS.UI.mm:SetWidth(36)
	
	OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self, h)
		if Onslaught_Settings.locked or MINIMAPDOCKER then
			local ui_x = UIParent:GetWidth()/2
			local ui_y = UIParent:GetHeight()/2
			
			local x = OS.UI.mm:GetLeft()
			local y = OS.UI.mm:GetTop()
			
			local sx,sy,tx,ty
			
			if x > ui_x then
				sx="RIGHT"
				tx="LEFT"
			else
				sx="LEFT"
				tx="RIGHT"
			end
			
			if y > ui_y then
				sy="BOTTOM"
				ty="TOP"
			else
				sy="TOP"
				ty="BOTTOM"
			end
			
			OS.UI.popup:ClearPoint("TOPLEFT")
			OS.UI.popup:ClearPoint("TOPRIGHT")
			OS.UI.popup:ClearPoint("BOTTOMRIGHT")
			OS.UI.popup:ClearPoint("BOTTOMLEFT")
			
			local sa = sy..sx
			local ta = ty..tx
			
			OS.UI.popup:SetPoint(sa, OS.UI.mm, ta, 0,0)
			OS.UI.popup:SetVisible(true)
		end
	end, "Event.UI.Input.Mouse.Cursor.In")

	OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self, h)
		if Onslaught_Settings.locked or MINIMAPDOCKER then
			OS.UI.popup:SetVisible(false)
		end
	end, "Event.UI.Input.Mouse.Cursor.Out")

	OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if MINIMAPDOCKER or Onslaught_Settings.locked then
			OS.HeaderClick()
		else
			self.MouseDown = true
			if Onslaught_Settings.locked == false then
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - OS.UI.mm:GetLeft()
				self.sy = mouseData.y - OS.UI.mm:GetTop()		
			else
				OS.HeaderClick()
			end
		end
	end, "Event.UI.Input.Mouse.Left.Down")
	
	if MINIMAPDOCKER == nil then
		OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			self.MouseDown = false
			if Onslaught_Settings.locked == false then
				Onslaught_Settings.mmx = OS.UI.mm:GetLeft()
				Onslaught_Settings.mmy = OS.UI.mm:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Up")

		OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if Onslaught_Settings.locked == false then
				if self.MouseDown then
					local nx, ny
					local mouseData = Inspect.Mouse()
					nx = mouseData.x - self.sx
					ny = mouseData.y - self.sy
					OS.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
				end
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")
		
		OS.UI.mm:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			self.MouseDown = false 
			Onslaught_Settings.locked = not Onslaught_Settings.locked
			if Onslaught_Settings.locked == false then
				OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_bw.png")
				OS.UI.popup:SetVisible(false)
			else
				if data.quests[_currentZone] ~= nil then
					OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_green.png")
				else
					OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_red.png")
				end
			end
		end, "Event.UI.Input.Mouse.Right.Click")
	end
	
	OS.UI.popup = UI.CreateFrame("Frame", "OS.UI.popup", OS.context)
	OS.UI.popup:SetVisible(false)
	OS.UI.popup:SetBackgroundColor(0,0,0,0.25)
	OS.UI.popup:SetLayer(10)
	OS.UI.popup:SetPoint("TOPLEFT", OS.UI.mm, "BOTTOMRIGHT", 0,0)
	
	OS.UI.header = UI.CreateFrame("Text", "OS.UI.header", OS.UI.popup)
	OS.UI.header:SetPoint("TOPLEFT", OS.UI.popup, "TOPLEFT", 0,0)
	OS.UI.header:SetFontSize(16)
	OS.UI.header:SetText("XXX")
	--OS.UI.header:SetBackgroundColor(0,0,0.5)
	OS.UI.header:SetFontColor(1,1,1)
	OS.UI.header_height = OS.UI.header:GetHeight()
	
	OS.UI.info = {}
	
	for x = 1, 25 do
		local f = UI.CreateFrame("Text", "OS.UI.info", OS.UI.popup)
		f:SetFontSize(12)
		f:SetFontColor(1,1,1)
		if x == 1 then
			f:SetPoint("TOPLEFT", OS.UI.header, "BOTTOMLEFT", 0,0)
			f:SetText("XXX")
			OS.UI.info_height = f:GetHeight()
		else
			f:SetPoint("TOPLEFT", OS.UI.info[x-1], "BOTTOMLEFT", 0,0)
		end
		f:SetVisible(false)
		OS.UI.info[x] = f
	end	
	
	OS.UI.gps = UI.CreateFrame("Frame", "OS.UI.gps", OS.context)
	OS.UI.gps:SetLayer(1)
	OS.UI.gps:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 600, 200)
	OS.UI.gps:SetBackgroundColor(0.25, 0.25, 0.25, 0.5)
	OS.UI.gps:SetVisible(false)
	OS.UI.gps_header = UI.CreateFrame("Text", "OS.UI.gps_header", OS.UI.gps)
	OS.UI.gps_header:SetLayer(2)
	OS.UI.gps_header:SetFontSize(16)
	OS.UI.gps_header:SetBackgroundColor(0,0.6,0)
	OS.UI.gps_header:SetText("ONSLAUGHT GPS")
	OS.UI.gps_header:SetPoint("TOPLEFT", OS.UI.gps, "TOPLEFT", 2,2)
	
	OS.UI.gps_mode = UI.CreateFrame("Text", "OS.UI.gps_mode", OS.UI.gps)
	OS.UI.gps_mode:SetLayer(3)
	OS.UI.gps_mode:SetFontSize(14)
	OS.UI.gps_mode:SetBackgroundColor(0,0.4,0)
	OS.UI.gps_mode:SetText("MODE: ")
	OS.UI.gps_mode:SetPoint("TOPLEFT", OS.UI.gps_header, "BOTTOMLEFT", 0,0)
	
	OS.UI.gps_mode:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		local lbl
		if Onslaught_Settings.gpsmode == GPS_MODE_AVAIL then
			Onslaught_Settings.gpsmode = GPS_MODE_ALL
			lbl = "MODE: ALL"
		else
			Onslaught_Settings.gpsmode = GPS_MODE_AVAIL
			lbl = "MODE: AVAILABLE"
		end
		OS.UI.gps_mode:SetText(lbl)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	OS.UI.gps_trk = UI.CreateFrame("Text", "OS.UI.gps_trk", OS.UI.gps)
	OS.UI.gps_trk:SetLayer(4)
	OS.UI.gps_trk:SetFontSize(14)
	OS.UI.gps_trk:SetBackgroundColor(0,0.4,0)
	OS.UI.gps_trk:SetText("QST")
	OS.UI.gps_trk:SetPoint("TOPRIGHT", OS.UI.gps_header, "BOTTOMRIGHT", 0,0)
	OS.UI.gps_mode:SetPoint("TOPRIGHT", OS.UI.gps_trk, "TOPLEFT", 0,0)
	
	OS.UI.gps_trk:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		local lbl
		if Onslaught_Settings.gpstrack == GPS_TRACK_QST then
			Onslaught_Settings.gpstrack = GPS_TRACK_OBJ
			lbl = "OBJ"
		else
			Onslaught_Settings.gpstrack = GPS_TRACK_QST
			lbl = "QST"
		end
		OS.UI.gps_trk:SetText(lbl)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	OS.UI.gps_dims = {}
	OS.UI.gps_dims.h_ht = OS.UI.gps_header:GetHeight()
	OS.UI.gps_dims.h_wd = OS.UI.gps_header:GetWidth()
	OS.UI.gps_dims.m_ht = OS.UI.gps_mode:GetHeight()
	
	OS.UI.gps:SetHeight(OS.UI.gps_dims.h_ht)
	OS.UI.gps:SetWidth(OS.UI.gps_dims.h_wd)

	OS.UI.gps:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - OS.UI.gps:GetLeft()
		self.sy = mouseData.y - OS.UI.gps:GetTop()		
	end, "Event.UI.Input.Mouse.Left.Down")
	
	OS.UI.gps:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			OS.UI.gps:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	OS.UI.gps:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		Onslaught_Settings.gpsx = OS.UI.gps:GetLeft()
		Onslaught_Settings.gpsy = OS.UI.gps:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")
	
	OS.UI.gps_tracker = {}
	
	OS.UI.gps_dims.i_ht = 0
	
	for x = 1, 25 do
		local t = {}
		t.f = UI.CreateFrame("Frame", "t.f", OS.UI.gps)
		t.f:SetLayer(2)
		local rgb = (x-1)/24
		t.i = UI.CreateFrame("Texture", "t.i", t.f)
		t.i:SetLayer(3)
		t.i:SetTexture(addon.identifier, "img/compass-N.png")
		t.i:SetPoint("TOPLEFT", t.f, "TOPLEFT", 0,0)
		t.n = UI.CreateFrame("Text", "t.n", t.f)
		t.n:SetLayer(3)
		t.n:SetFontSize(12)
		t.n:SetText("NAME")
		t.n:SetPoint("TOPLEFT", t.i, "TOPRIGHT", 0,0)
		t.d = UI.CreateFrame("Text", "t.n", t.f)
		t.d:SetLayer(3)
		t.d:SetFontSize(8)
		t.d:SetText("DISTANCE")
		t.d:SetPoint("BOTTOMLEFT", t.i, "BOTTOMRIGHT", 0,0)
		if OS.UI.gps_dims.i_ht == 0 then
			OS.UI.gps_dims.n_ht = t.n:GetHeight()
			OS.UI.gps_dims.d_ht = t.d:GetHeight()
			OS.UI.gps_dims.i_ht = OS.UI.gps_dims.n_ht + OS.UI.gps_dims.d_ht
		end
		t.i:SetWidth(OS.UI.gps_dims.i_ht)
		t.i:SetHeight(OS.UI.gps_dims.i_ht)
		t.f:SetHeight(OS.UI.gps_dims.i_ht)
	
		if x == 1 then
			t.f:SetPoint("TOPLEFT", OS.UI.gps_mode, "BOTTOMLEFT", 0,0)
		else
			t.f:SetPoint("TOPLEFT", OS.UI.gps_tracker[x-1].f, "BOTTOMLEFT", 0,0)
		end
		
		OS.UI.gps_tracker[x] = t
	end
	
	OS.UI.gps_config = UI.CreateFrame("Frame", "OS.UI.gps_config", OS.UI.gps)
	OS.UI.gps_config:SetVisible(false)
	OS.UI.gps_config:SetLayer(10)
	OS.UI.gps_config:SetHeight(OS.UI.gps_header:GetHeight())
	OS.UI.gps_config:SetPoint("TOPLEFT", OS.UI.gps, "TOPLEFT", 2,2)
	OS.UI.gps_config:SetBackgroundColor(0.6,0,0)
	OS.UI.gps_config.visible = false
	
	OS.UI.gps_config:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		OS.UI.gps_config:SetVisible(false)
	end, "Event.UI.Input.Mouse.Right.Click")

	OS.UI.gps_header:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		OS.UI.gps_config:SetVisible(true)
	end, "Event.UI.Input.Mouse.Right.Click")
	
	OS.UI.gps_cfg_lbl = UI.CreateFrame("Text", "OS.UI.gps_cfg_lbl", OS.UI.gps_config)
	OS.UI.gps_cfg_lbl:SetLayer(12)
	OS.UI.gps_cfg_lbl:SetFontSize(16)
	OS.UI.gps_cfg_lbl:SetText("LABEL")
	OS.UI.gps_cfg_lbl:SetPoint("TOPLEFT", OS.UI.gps_config, "TOPLEFT")
	
	OS.UI.gps_cfg_lbl:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Onslaught_Settings.fonts.label < 36 then
			Onslaught_Settings.fonts.label = Onslaught_Settings.fonts.label+1
			OS.GPS_Scale()
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	OS.UI.gps_cfg_lbl:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Onslaught_Settings.fonts.label > 4 then
			Onslaught_Settings.fonts.label = Onslaught_Settings.fonts.label - 1
			OS.GPS_Scale()
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")

	OS.UI.gps_cfg_lbl:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		OS.UI.gps_config:SetVisible(false)
	end, "Event.UI.Input.Mouse.Right.Click")

	OS.UI.gps_cfg_rng = UI.CreateFrame("Text", "OS.UI.gps_cfg_rng", OS.UI.gps_config)
	OS.UI.gps_cfg_rng:SetLayer(12)
	OS.UI.gps_cfg_rng:SetFontSize(16)
	OS.UI.gps_cfg_rng:SetText("DISTANCE")
	OS.UI.gps_cfg_rng:SetPoint("TOPRIGHT", OS.UI.gps_config, "TOPRIGHT")

	OS.UI.gps_cfg_rng:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Onslaught_Settings.fonts.dist < 36 then
			Onslaught_Settings.fonts.dist = Onslaught_Settings.fonts.dist+1
			OS.GPS_Scale()
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	OS.UI.gps_cfg_rng:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Onslaught_Settings.fonts.dist > 4 then
			Onslaught_Settings.fonts.dist = Onslaught_Settings.fonts.dist - 1
			OS.GPS_Scale()
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")

	OS.UI.gps_cfg_rng:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		OS.UI.gps_config:SetVisible(false)
	end, "Event.UI.Input.Mouse.Right.Click")
end

function OS.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if Onslaught_Settings == nil then Onslaught_Settings = {day="", complete={}} end
		MergeTable(Onslaught_Settings, default_settings)
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, OS.UI.mm)
		else
			OS.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Onslaught_Settings.mmx, Onslaught_Settings.mmy)
		end
		OS.UI.gps:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Onslaught_Settings.gpsx, Onslaught_Settings.gpsy)
		if Onslaught_Settings.autoswap then
			OS.UI.header:SetBackgroundColor(0.5,0,0)
		else
			OS.UI.header:SetBackgroundColor(0,0,0.5)
		end
		local lbl = "MODE: AVAILABLE"
		if Onslaught_Settings.gpsmode == GPS_MODE_ALL then
			lbl = "MODE: ALL"
		end
		OS.UI.gps_mode:SetText(lbl)
		lbl = "QST"
		if Onslaught_Settings.gpstrack == GPS_TRACK_OBJ then
			lbl = "OBJ"
		end
		OS.UI.gps_trk:SetText(lbl)
		OS.GPS_Scale()
		OS.CheckDay()
	end
end

function OS.Event_Quest_Complete(h,t)
	INSPECT_QUEST_LIST = Inspect.Quest.List()
	for k,v in pairs(t) do
		if _QID_ALL[k] ~= nil then
			OS.CheckDay()
			Onslaught_Settings.complete[k] = true
			OS.UpdateUI()
		end
	end
end

function OS.Event_Quest_Accept(h,t)
	INSPECT_QUEST_LIST = Inspect.Quest.List()
	for k,v in pairs(t) do
		if _QID_ALL[k] ~= nil then
			OS.UpdateUI()
		end
	end
end

function OS.Event_Quest_Abandon(h,t)
	INSPECT_QUEST_LIST = Inspect.Quest.List()
	for k,v in pairs(t) do
		if _QID_ALL[k] ~= nil then
			OS.UpdateUI()
		end
	end
end

local ui_i = 0
local ui_c = 0
local ui_w = 0

function OS.UpdateUI()
	if Onslaught_Settings.locked == false and MINIMAPDOCKER == nil then
		OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_bw.png")
	else
		if data.quests[_currentZone] ~= nil then
			OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_green.png")
		else
			OS.UI.mm:SetTexture(addon.identifier, "img/onslaught_red.png")
		end
	end

	if data.quests[_currentZone] ~= nil then
		
		OS.UI.header:ClearWidth()
		OS.UI.header:SetText(Inspect.Zone.Detail(_currentZone).name or "UNKNOWN")
		ui_i = 0
		ui_c = 0
		ui_w = OS.UI.header:GetWidth()
		
		for k,v in pairs(data.quests[_currentZone].quests) do
			ui_i = ui_i + 1
			OS.UI.info[ui_i]:ClearWidth()
			if _QID_DTL[k] == nil then
				OS.UI.info[ui_i]:SetText(string.format("+%s", v))
				if _QID_ONS.list[k] == nil then
					_QID_ONS.cnt =_QID_ONS.cnt +1
					_QID_ONS.list[k] = true
					ONS_QSTDTL = true
				end
			else
				OS.UI.info[ui_i]:SetText(_QID_DTL[k].name)
			end
			OS.UI.info[ui_i]:SetVisible(true)
			ui_w = math_max(ui_w, OS.UI.info[ui_i]:GetWidth())
			if Onslaught_Settings.complete[k] ~= nil then
				OS.UI.info[ui_i]:SetBackgroundColor(0,0.5,0)
				ui_c = ui_c + 1
			else
				if INSPECT_QUEST_LIST ~= nil then
					if INSPECT_QUEST_LIST[k] ~= nil then
						OS.UI.info[ui_i]:SetBackgroundColor(0,0.25,0)
					else
						OS.UI.info[ui_i]:SetBackgroundColor(0,0,0)
					end
				else
					OS.UI.info[ui_i]:SetBackgroundColor(0,0,0)
				end
			end
		end
		
		OS.UI.header:SetWidth(ui_w)
		OS.UI.popup:SetWidth(ui_w)
		OS.UI.popup:SetHeight(OS.UI.header_height + (OS.UI.info_height * ui_i))
		for x=1, ui_i do
			OS.UI.info[x]:SetWidth(ui_w)
		end
		for x=ui_i+1, 25 do
			OS.UI.info[x]:SetVisible(false)
		end
		if GPS_VISIBLE == false and Onslaught_Settings.usegps then
			OS.UI.gps:SetVisible(true)
			GPS_VISIBLE = true
		end		
	else
		OS.UI.header:ClearWidth()
		OS.UI.header:SetText("No Onslaught in this zone")
		OS.UI.popup:SetHeight(OS.UI.header_height)
		OS.UI.popup:SetWidth(OS.UI.header:GetWidth())
		for x=1, 25 do
			OS.UI.info[x]:SetVisible(false)
		end
		
		if GPS_VISIBLE then
			OS.UI.gps:SetVisible(false)
			GPS_VISIBLE = false
		end
	end
end

function OS.Event_libZoneChange_Player(h, zn, zid)
	_currentZone = zid
	OS.HandleZoneChange()
end

function OS.HandleZoneChange()
	INSPECT_QUEST_LIST = Inspect.Quest.List()
	if Onslaught_Settings.autoswap then
		local showit = false
		for k,v in pairs(INSPECT_QUEST_LIST) do
			if _QID_ALL[k] then
				showit = false
				if data.quests[_currentZone] then
					if data.quests[_currentZone].quests[k] then
						showit = true
					end
				end
				Command.Quest.Watch(k, showit)
			end
		end
	end

	if data.quests[_currentZone] then
		for k,v in pairs(data.quests[_currentZone].quests) do
			_QID_ONS.list[k] = true
			_QID_ONS.cnt = _QID_ONS.cnt + 1
		end
		ONS_QSTDTL = true
	else
		OS.UpdateUI()
	end
	
	if data.quests[_currentZone] and Onslaught_Settings.usegps then
		OS.UI.gps:SetVisible(true)
		GPS_VISIBLE = true
	else
		OS.UI.gps:SetVisible(false)
		GPS_VISIBLE = false
	end	
end

local uh_fw = 0
local uh_cnw = 0

local oldw = 0

function OS.UpdateHandler()
	while true do
		if ONS_QSTDTL then
			if _QID_ONS.cnt > 0 then
				for k,v in pairs(_QID_ONS.list) do
					if _QID_DTL[k] == nil then
						local wd = Inspect.System.Watchdog()
						while wd < 0.03 do
							coroutine.yield()
						end
						INSPECT_QUEST_DETAIL = Inspect.Quest.Detail(k)
						if INSPECT_QUEST_DETAIL ~= nil then
							if string.byte(INSPECT_QUEST_DETAIL.name, string.len(INSPECT_QUEST_DETAIL.name)) == 10 then
								_QID_DTL[k] = { name = string.sub(INSPECT_QUEST_DETAIL.name, 1, -2) }
							else
								_QID_DTL[k] = { name = INSPECT_QUEST_DETAIL.name }
							end
							_QID_ONS.cnt = _QID_ONS.cnt - 1
							_QID_ONS.list[k] = nil
						end
					else
						_QID_ONS.cnt = _QID_ONS.cnt - 1
						_QID_ONS.list[k] = nil
					end
				end
				if _QID_ONS.cnt == 0 then
					ONS_QSTDTL = false
					OS.UpdateUI()
				end
			else
				ONS_QSTDTL = false
			end
		end
		if ONS_GPSUPDATE then
			OS.GPS()
			uh_fw = OS.UI.gps_dims.h_wd
			for k,v in pairs(GPSDATA) do
				OS.UI.gps_tracker[k].i:SetTexture(addon.identifier, string_format("img/compass-%s.png", v.c))
				OS.UI.gps_tracker[k].n:SetText(v.n or "UNKNOWN")
				if v.cx then
					OS.UI.gps_tracker[k].n:SetFontColor(1,0,0)
				elseif (INSPECT_QUEST_LIST and INSPECT_QUEST_LIST[v.q]) then
					OS.UI.gps_tracker[k].n:SetFontColor(0,1,0)
				else
					OS.UI.gps_tracker[k].n:SetFontColor(1,1,1)
				end
				uh_fw = math_max(uh_fw, math.max(OS.UI.gps_tracker[k].n:GetWidth(), OS.UI.gps_tracker[k].d:GetWidth())+OS.UI.gps_dims.i_ht)
				OS.UI.gps_tracker[k].d:SetText(string_format("%dm", v.d or 9999))
				OS.UI.gps_tracker[k].f:SetVisible(true)
			end
			OS.UI.gps:SetWidth(4+uh_fw)
			OS.UI.gps_header:SetWidth(uh_fw)
			OS.UI.gps_config:SetWidth(uh_fw)
			OS.UI.gps:SetHeight(4+(#GPSDATA*(OS.UI.gps_dims.n_ht+OS.UI.gps_dims.d_ht))+OS.UI.gps_dims.h_ht+OS.UI.gps_dims.m_ht)
			for x=#GPSDATA+1, 25 do
				OS.UI.gps_tracker[x].f:SetVisible(false)
			end
			GPSDXCNT = #GPSDATA
			ONS_GPSUPDATE = false
		end
		coroutine.yield()
	end
end

OS.coUpdateHandler = coroutine.create(OS.UpdateHandler)

local last_updateUI = Inspect.Time.Real()
local last_updateGPS= Inspect.Time.Real()
local _updatecount = 0
local cur_update = 0

function OS.Event_System_Update_Begin(h)
	cur_update = Inspect.Time.Real()
	if (cur_update - last_updateUI ) > 60 then
		OS.CheckDay()
		last_updateUI = cur_update
	end
	if ((cur_update - last_updateGPS) > 1) and GPS_VISIBLE then
		last_updateGPS = cur_update
		ONS_GPSUPDATE = true
	end
	_updatecount = _updatecount+1
	if _updatecount == 100 then
		_updatecount = 0
		OS.UpdateUI()
	end
	if coroDead then return end
	local ok, errormsg = coroutine.resume(OS.coUpdateHandler)
	if not ok then
		coroDead = true
		error("Error in coroutine: "..errormsg)
	end
end

function OS.ShowCommandOptions()
	print(string_format("/onslaught autoswap - toggles autoswap of quests on zone change"))
end

function OS.direction(dt)
	if dt > 180 then dt = dt-360 end
	if dt < -180 then dt = dt+360 end

	if dt >= 169		then	d = "S"
	elseif dt >= 146	then	d = "SSW"
	elseif dt >= 124	then	d = "SW"
	elseif dt >= 101	then	d = "WSW"
	elseif dt >= 79		then	d = "W"
	elseif dt >= 56		then	d = "WNW"
	elseif dt >= 34		then	d = "NW"
	elseif dt >= 11		then	d = "NNW"
	elseif dt >= -11	then	d = "N"
	elseif dt >= -34	then	d = "NNE"
	elseif dt >= -56	then	d = "NE"
	elseif dt >= -79	then	d = "ENE"
	elseif dt >= -101	then	d = "E"
	elseif dt >= -124	then	d = "ESE"
	elseif dt >= -146	then	d = "SE"
	elseif dt >= -169	then	d = "SSE"
	else						d = "S"
	end

	return {i="", l=d}
end

function OS.GPS()
	GPSDATA = {}
	if data.quests[_currentZone] then
		INSPECT_UNIT_DETAIL = Inspect.Unit.Detail("player")
		if INSPECT_UNIT_DETAIL.coordX then
			for k,v in pairs(data.quests[_currentZone].quests) do
				local qd = data.locations[k]
				if qd then
					local trkqd
					if Onslaught_Settings.gpstrack == GPS_TRACK_QST then
						trkqd = qd.q
					else
						trkqd = qd.o
					end
					local dx = INSPECT_UNIT_DETAIL.coordX - trkqd.x
					local dz = INSPECT_UNIT_DETAIL.coordZ - trkqd.z
					local d = math_sqrt((dx^2)+(dz^2))
					local dr = math_atan(dx,dz)*180/math_pi
					local c = OS.direction(dr).l
					if Onslaught_Settings.complete[k] == nil then
						table.insert(GPSDATA, {q=k, n=v, d=d, c=c, cx = false})
					else
						if Onslaught_Settings.gpsmode == GPS_MODE_ALL then
							table.insert(GPSDATA, {q=k, n=v, d=d, c=c, cx = true})
						end
					end
				end
			end
			table.sort(GPSDATA, function(a,b) return a.d < b.d end)
		end
	end
end

function OS.DataCheck()
	for k,v in pairs(data.quests) do
		for k1,v1 in pairs(v.quests) do
			if data.locations[k1] == nil then
				print(string.format("MISSING: %s [%s]", v1, v.name))
			end
		end
	end
end

function OS.Command_Slash_Register(h, args)
	local r = {}
	local numargs = 0
	for token in string.gmatch(args, "[^%s]+") do
		numargs=numargs+1
		r[numargs] = token
	end
	if numargs>0 then
		if r[1] == "autoswap" then
			Onslaught_Settings.autoswap = not Onslaught_Settings.autoswap
			if Onslaught_Settings.autoswap then
				OS.UI.header:SetBackgroundColor(0.5,0,0)
				print("Toggling quest visibility on zone change")
			else
				OS.UI.header:SetBackgroundColor(0,0,0.5)
				print("NOT toggling quest visibility on zone change")
			end		
		elseif r[1] == "gps" then
			OS.GPS()
		elseif r[1] == "check" then
			OS.DataCheck()
		else
			OS.ShowCommandOptions()
		end
	else
		OS.ShowCommandOptions()
	end
end

for k,_ in pairs(data.quests) do
	for q,n in pairs(data.quests[k].quests) do
		_QID_ALL[q] = true
	end
end

ONS_QSTDTL = true

OS.BuildUI()

Command.Event.Attach(Event.System.Update.Begin, OS.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Event.Quest.Complete, OS.Event_Quest_Complete, "Event.Quest.Complete")
Command.Event.Attach(Event.Quest.Accept, OS.Event_Quest_Accept, "Event.Quest.Accept")
Command.Event.Attach(Event.Quest.Abandon, OS.Event_Quest_Abandon, "Event.Quest.Abandon")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, OS.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Command.Slash.Register("onslaught"), OS.Command_Slash_Register, "Command.Slash.Register")
Command.Event.Attach(Library.libZoneChange.Player, OS.Event_libZoneChange_Player, "Library.LibZoneChange.Player")

print(string_format("v%s loaded.", addon.toc.Version))